@@ -14,17 +14,19 @@ def circle_aperture(field, radius=1):
     """
     Generate a circle aperture.
 
-    Args:
-        field: tuple
+    Parameters
+    ----------
+        field : SimLight.Field
             Input light field.
-        radius: float
-            Normalized radius of the circle. (optional)
+        radius : float, optional, default 1
+            Normalized radius of the circle.
     """
     field = sl.Field.copy(field)
 
     x = np.linspace(-field.size / 2, field.size / 2, field.N)
     X, Y = np.meshgrid(x, x)
     R = np.sqrt(X**2 + Y**2)
-    field.complex_amp[R >= radius * field.size / 2] = 0
+    field.complex_amp[R > radius * field.size / 2] = 0
+    field.complex_amp2[R > radius * field.size / 2] = 0
 
     return field
